//#region rolldown:runtime
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __copyProps = (to, from, except, desc) => {
	if (from && typeof from === "object" || typeof from === "function") for (var keys = __getOwnPropNames(from), i = 0, n = keys.length, key; i < n; i++) {
		key = keys[i];
		if (!__hasOwnProp.call(to, key) && key !== except) __defProp(to, key, {
			get: ((k) => from[k]).bind(null, key),
			enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable
		});
	}
	return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", {
	value: mod,
	enumerable: true
}) : target, mod));

//#endregion
const require_utils = require('./utils-pR74ehGT.cjs');
const unplugin = __toESM(require("unplugin"));
const node_path = __toESM(require("node:path"));
const debug = __toESM(require("debug"));
const fast_glob = __toESM(require("fast-glob"));
const node_fs = __toESM(require("node:fs"));

//#region src/module/files.ts
const debug$3 = (0, debug.default)("inertia-page-loader:module:files");
function getIgnore(exclude) {
	return [
		".git",
		"**/__*__/**",
		...exclude
	];
}
function extsToGlob(extensions) {
	return extensions.length > 1 ? `{${extensions.join(",")}}` : extensions[0] || "";
}
function getPageFiles(path$2, options = {}) {
	const { extensions = [""], exclude = [] } = options;
	const ext = extsToGlob(extensions);
	const files = fast_glob.default.sync((0, node_path.join)(path$2, `**/*.${ext}`).replaceAll("\\", "/"), {
		cwd: options.cwd,
		ignore: getIgnore(exclude),
		onlyFiles: true
	});
	debug$3(files);
	return files;
}

//#endregion
//#region src/module/import.ts
const debug$2 = {
	pages: (0, debug.default)("inertia-page-loader:module:import:pages"),
	imports: (0, debug.default)("inertia-page-loader:module:import:imports")
};
function generateImportGlobCode(pattern, options = {}) {
	const { eager = false } = options;
	let start = options.start ?? 0;
	const files = getPageFiles(pattern, options).map((file) => {
		file = file.replace(/^(\/|\.\/)/, "");
		if (!file.startsWith(".")) file = `/${file}`;
		return file;
	});
	const imporetedModules = {};
	let pages = files.map((file) => {
		if (eager) {
			imporetedModules[file] = `__import_page_${start}__`;
			start++;
			return `            "${file}": () => import(${imporetedModules[file]}),\n`;
		}
		return `            "${file}": () => import("${file}"),\n`;
	}).join("");
	pages = `{\n${pages}          }`;
	debug$2.pages(pages);
	let imports = "";
	if (eager) imports = files.map((file) => `import ${imporetedModules[file]} from '${file}'`).join("\n");
	debug$2.imports(imports);
	return {
		imports,
		pages,
		start
	};
}

//#endregion
//#region src/page/namespace-option.ts
function createPackageNamespaceExtractor(options) {
	return (pkg, dir = options.dir) => {
		const fullpath = node_path.default.resolve(options.cwd, dir, pkg, options.filename);
		if (!node_fs.default.existsSync(fullpath)) throw new Error(`[inertia-page-loader]: The ${options.name} "${pkg}" does not exist`);
		const namespaces = options.parse(node_fs.default.readFileSync(fullpath, { encoding: "utf-8" }));
		if (!namespaces) throw new Error(`[inertia-page-loader]: The ${options.filename} parse error of "${pkg}"`);
		for (const namespace in namespaces) {
			const mod = namespaces[namespace];
			if (Array.isArray(mod)) namespaces[namespace] = mod.map((mod$1) => node_path.default.join(dir, pkg, mod$1).replaceAll("\\", "/"));
			else namespaces[namespace] = node_path.default.join(dir, pkg, mod).replaceAll("\\", "/");
		}
		return namespaces;
	};
}
function createNpm(cwd = process.cwd()) {
	return createPackageNamespaceExtractor({
		name: "NPM package",
		filename: "package.json",
		cwd,
		dir: "node_modules",
		parse(content) {
			return JSON.parse(content).inertia;
		}
	});
}
function createComposer(cwd = process.cwd()) {
	return createPackageNamespaceExtractor({
		name: "Composer package",
		filename: "composer.json",
		cwd,
		dir: "vendor",
		parse(content) {
			return JSON.parse(content).extra.inertia;
		}
	});
}
function resolveNamespaces(cwd, namespaces) {
	const resolvedNamespaces = typeof namespaces === "function" ? namespaces({
		npm: createNpm(cwd),
		composer: createComposer(cwd)
	}) : namespaces;
	const output = {};
	for (const obj of resolvedNamespaces) for (const key of Object.keys(obj)) output[key] = Array.isArray(obj[key]) ? obj[key] : [obj[key]];
	return output;
}

//#endregion
//#region src/page/generate-namespaces.ts
const debug$1 = (0, debug.default)("inertia-page-loader:page:generate-namespaces");
function generateNamespacesCode(options, meta) {
	const cwd = options.cwd;
	const namespaces = resolveNamespaces(cwd, options.namespaces);
	let importsCode = "";
	let importStartNum = 0;
	let namespacesCode = Object.keys(namespaces).map((namespace) => {
		const modules = namespaces[namespace];
		let code = `        '${namespace}': [\n`;
		if (require_utils.isViteLike(meta.framework)) modules.forEach((moduleDir) => {
			const { imports, pages, start } = generateImportGlobCode(moduleDir, {
				start: importStartNum,
				eager: options.ssr,
				cwd,
				extensions: options.extensions
			});
			importStartNum = start;
			importsCode += `${importsCode ? "\n" : ""}${imports}`;
			code += `          name => resolveVitePage(name, ${pages}, false),\n`;
		});
		else if (require_utils.isWebpackLike(meta.framework)) modules.forEach((moduleDir) => {
			const moduleImporter = options.ssr || !options.ssr && !options.import ? "require" : "import";
			const extension = options.extensions[0] ? `.${options.extensions[0]}` : "";
			let modulePath = node_path.default.relative(cwd, node_path.default.resolve(cwd, moduleDir, `\${name}${extension}`)).replace(/\\/g, "/");
			if (!modulePath.startsWith(".")) modulePath = `./${modulePath}`;
			code += `          name => ${moduleImporter}(\`${modulePath}\`),\n`;
		});
		code += "        ],\n";
		return code;
	}).join("");
	namespacesCode = `{\n${namespacesCode}      }`;
	debug$1(namespacesCode);
	debug$1(importsCode);
	return {
		namespacesCode,
		importsCode
	};
}

//#endregion
//#region src/page/index.ts
function pageLoader(options, meta) {
	const { namespacesCode, importsCode } = generateNamespacesCode(options, meta);
	return `
${importsCode}

export function resolvePage(resolver, transformPage) {
  return async name => {
    let page = await resolvePluginPage(name)
    if (!page) {
      page = ${require_utils.isViteLike(meta.framework) ? "await resolveVitePage(name, await resolver(name.replace('.', '/')))" : "await resolver(name.replace('.', '/'))"}
    }
    page = page.default || page
    if (transformPage) {
      page = transformPage(page, name)
    }
    return page
  }
}

export async function resolvePluginPage(name) {
  if (name.includes('${options.separator}')) {
    const [namespace, page] = name.split('${options.separator}')
    const meta = { framework: '${meta.framework}' }

    if (namespace && page) {
      const namespaces = ${namespacesCode}

      /* Load namespaces on runtime from window global variable. */
      if (window.InertiaPages) {
        for (const namespaceGroup of window.InertiaPages.namespaces) {
          for (const namespace in namespaceGroup) {
            namespaces[namespace] = (namespaces[namespace] || []).concat(namespaceGroup[namespace])
          }
        }
      }

      if (!namespaces[namespace]) {
        throw new Error(\`[inertia-page-loader]: Namespace "\${namespace}" not found\`)
      }

      for (const importedNamespace of namespaces[namespace]) {
        if (importedNamespace && typeof importedNamespace === 'function') {
          return await importedNamespace(page, meta)
        }
      }
    }
  }
}

export async function resolveVitePage(name, pages, throwNotFoundError = true) {
  for (const path in pages) {
    if (${JSON.stringify(options.extensions)}.some(ext => path.endsWith(\`\${name.replaceAll('.', '/')}.\${ext}\`))) {
      const module = typeof pages[path] === 'function'
        ? pages[path]()
        : pages[path]

      return await Promise.resolve(module).then(module => module.default || module)
    }
  }

  if (throwNotFoundError) {
    throw new Error(\`[inertia-page-loader]: Page "\${name}" not found\`)
  }
}`;
}

//#endregion
//#region src/index.ts
const ids = [
	"/~inertia",
	"~inertia",
	"virtual:inertia",
	"virtual/inertia"
];
function resolveOptions(options, meta) {
	let extensions = options.extensions;
	if (require_utils.isViteLike(meta.framework) && !extensions) extensions = "vue";
	else if (require_utils.isWebpackLike(meta.framework) && Array.isArray(extensions)) extensions = extensions[0];
	extensions = Array.isArray(extensions) ? extensions : [extensions ?? ""];
	extensions = extensions.map((ext) => ext.replace(/^\./, ""));
	return Object.assign({
		cwd: process.cwd(),
		namespaces: [],
		separator: "::",
		extensions: [""],
		import: false,
		ssr: false
	}, options, { extensions });
}
const unpluginFactory = (userOptions, meta) => {
	const options = resolveOptions(userOptions || {}, meta);
	return {
		name: "inertia-page-loader",
		enforce: "pre",
		resolveId(id) {
			if (ids.includes(id)) return ids[1];
		},
		load(id) {
			if (ids.includes(id)) {
				const code = pageLoader(options, meta);
				return {
					code,
					map: {
						version: 3,
						mappings: "",
						sources: []
					}
				};
			}
		}
	};
};
const unplugin$1 = /* @__PURE__ */ (0, unplugin.createUnplugin)(unpluginFactory);
var src_default = unplugin$1;

//#endregion
Object.defineProperty(exports, '__toESM', {
  enumerable: true,
  get: function () {
    return __toESM;
  }
});
Object.defineProperty(exports, 'src_default', {
  enumerable: true,
  get: function () {
    return src_default;
  }
});
Object.defineProperty(exports, 'unplugin', {
  enumerable: true,
  get: function () {
    return unplugin$1;
  }
});
Object.defineProperty(exports, 'unpluginFactory', {
  enumerable: true,
  get: function () {
    return unpluginFactory;
  }
});