import { UnpluginContextMeta } from "unplugin";

//#region rolldown:runtime
declare namespace plugin_d_exports {
  export { InertiaPages as default };
}
declare class InertiaPages {
  private _namespaces;
  addNamespace(namespace: string, resolver: PageResolver | PageResolver[]): this;
  get namespaces(): Record<string, PageResolver | PageResolver[]>[];
}
//#endregion
//#region src/types.d.ts
declare global {
  var InertiaPages: InstanceType<typeof plugin_d_exports['default']>;
}
interface Options {
  /**
   * Current work directory.
   *
   * @default process.cwd()
   */
  cwd?: string;
  /**
   * Define namespace mapping.
   *
   * @default []
   */
  namespaces?: Namespaces;
  /**
   * Namespace separator.
   *
   * @default '::'
   */
  separator?: string;
  /**
   * Module extensions.
   *
   * vite:
   * @type {string|string[]}
   * @default 'vue'
   *
   * webpack:
   * @type {string}
   * @default ''
   */
  extensions?: string | string[];
  /**
   * Use `import()` to load pages for webpack, default is using `require()`.
   * Only for webpack.
   *
   * @default false
   */
  import?: boolean;
  /**
   * Enable SSR mode.
   *
   * @default false
   */
  ssr?: boolean;
}
type ResolvedOptions = Required<Omit<Options, 'extensions' | 'extension'>> & {
  /**
   * Module extensions.
   */
  extensions: string[];
};
type PageResolver<T = any> = (name: string) => T | Promise<T>;
type Namespace = Record<string, string | string[]>;
type ResolvedNamespace = Record<string, string[]>;
interface NamespacesArgs {
  npm: (pkg: string, dir: string) => Namespace;
  composer: (pkg: string, dir: string) => Namespace;
}
type Namespaces = Namespace[] | ((args: NamespacesArgs) => Namespace[]);
interface PackageNamespaceExtractor {
  (pkg: string, dir?: string): Namespace;
}
interface GenerateNamespacesCodeContextMeta extends Pick<UnpluginContextMeta, 'framework'> {}
//#endregion
export { GenerateNamespacesCodeContextMeta, Namespace, Namespaces, NamespacesArgs, Options, PackageNamespaceExtractor, PageResolver, ResolvedNamespace, ResolvedOptions };